<?php
/**
* Plugin Name: Permafrost
* Short Name: Permafrost
* Description: Make Permalinks permanently. Used for fixing purposes.
* Author: hakre
* Version: 0.4
* Requires at least: 2.7
* Tested up to: 2.9-bleeding-edge
* Tags: permalink, permalinks, feed, feeds, debug, debugging, error, errors, problems, support, admin, programmer, developer, plugin, development
* Contributors: hakre
* Author URI: http://www.subfiles.net/
* Plugin URI: http://www.subfiles.net/wordpress/plugins/permafrost/
*
* Permafrost - Permanent Permalinks
* Copyright (C) 2009	hakre
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.	If not, see <http://www.gnu.org/licenses/>.
*/

class pluginPermafrost
{
	/**
	 * static instance of plugin
	 * 
	 * @var pluginPermafrost 
	 */
	static $__staticinstance = null;
	
	/**
	 * bootstrap plugin
	 */
	static function bootstrap()
	{
		if (is_null(pluginPermafrost::$__staticinstance))
		{
			pluginPermafrost::$__staticinstance = new pluginPermafrost();
		}
	}
	
	/**
	 * default filter priority
	 * 
	 * @var integer
	 */
	private $_priority = -1000;
	
	
	/**
	 * WordPress Filter ID - used to properly work with add_filter and remove_filter.
	 * 
	 * @var string
	 */
	public $wp_filter_id = 'pluginPermafrostFilterID';

	/**
	 * getter for the global WP_Rewrite
	 * 
	 * @return WP_Rewrite
	 */
	private function _getWP_Write()
	{
		global $wp_rewrite; // import $wp_rewrite from the global namespace
		
		// if not set, construct instance
		if (!isset($wp_rewrite))
		{			
			$wp_rewrite = new WP_Rewrite();			
		}

		return $wp_rewrite;
	}
	
	/**	
	 * output debug data
	 */
	private function _bumbCry()
	{
		global $wp_rewrite;
		
		// check if we are queried on / after cron
		var_dump($wp_rewrite);
		print '<pre>';
		debug_print_backtrace();
		print '</pre>';
	} 
	

	/**
	 * constructor
	 */
	public function __construct()
	{
		// do_action('setup_theme');
		// add_filter('setup_theme', array($this, 'rebuildPermalinks'), -10000);
		// add_filter('pre_get_posts', array($this, 'rebuildPermalinksOnce'), -10000);
		
		// hook in as early as possible
		add_filter('pre_option_permalink_structure', array($this, 'hookOption'), $this->_priority, 1);		
	}
	
	/**
	 * hookOption
	 * 
	 * function to hook in the pre_option_permalink_structure filter
	 * this is the earliest possible point to rebuild the permalink structure
	 * 
	 * @param  bool $bool boolean value 
	 * @return bool input value (normally false)
	 */
	public function hookOption($bool)
	{
		remove_filter('pre_option_permalink_structure', array($this, 'hookOption'), $this->_priority, 1);		
		
		$this->_getWP_Write()->set_permalink_structure($permalink_structure = get_option('permalink_structure'));			
				
		return $bool;		
	}
	
	/**
	 * rebuildPermalinks
	 * 
	 * command wordpress to rebuild permalink structure
	 */
	public function rebuildPermalinks()
	{		
		$this->_bumbCry();
		
		$this->_getWP_Write()->set_permalink_structure($permalink_structure = get_option('permalink_structure'));
	}
	
	/**
	 * rebuildPermalinksOnce
	 * 
	 * command wordpress to rebuild permalink structure - but only once per request
	 */
	public function rebuildPermalinksOnce()
	{		
		static $count = 0;
		
		if (!$count)
		{				
			$count++;
			
			$this->rebuidPermalinks();
		}
	}
	
}

pluginPermafrost::bootstrap();
?>